@@ -168,6 +168,7 @@ DiskManagerAllocationScheduler
 			CoreOperationTask task =
 				new CoreOperationTask()
 				{
+					@Override
 					public String
 					getName()
 					{
@@ -181,11 +182,20 @@ DiskManagerAllocationScheduler
 						return( helper.getDownload());
 					}
 					
+					@Override
 					public ProgressCallback
 					getProgressCallback()
 					{
 						return( progress );
 					}
+					
+					@Override
+					public boolean 
+					runOperation(
+						CoreOperation operation)
+					{
+						return( true );
+					}
 				};
 				
 			operation = 
