@@ -44,6 +44,7 @@ import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.core.util.*;
 import com.biglybt.core.util.FileUtil.ProgressListener;
+import com.biglybt.core.util.StringInterner.FileKey;
 
 import java.io.File;
 import java.io.IOException;
@@ -246,7 +247,7 @@ DiskManagerUtil
 								// these are compact files and can't be recovered as they just contain
 								// start/end piece data
 							
-							if ( currentFile.exists()){
+							if ( FileUtil.exists( currentFile )){
 								
 								FileUtil.log( "Deleting \"" + currentFile.getAbsolutePath() + "\" as no longer required", true );
 
@@ -288,7 +289,7 @@ DiskManagerUtil
 		    	
 		    	if ( !FileUtil.areFilePathsIdentical( to_link, existing_file )){
 	        		
-		    		if ( existing_file.exists()){
+		    		if ( FileUtil.exists( existing_file )){
 		    			
 			            if ( !FileUtil.renameFile( existing_file, to_link )){
 	
@@ -314,9 +315,9 @@ DiskManagerUtil
 			        }
 			    }
 	
-			    if ( to_link.exists()){
+			    if ( FileUtil.exists( to_link )){
 	
-			        if ( !existing_file.exists()){
+			        if ( !FileUtil.exists( existing_file )){
 	
 			                // using a new file, make sure we recheck
 	
@@ -378,7 +379,7 @@ DiskManagerUtil
 			        }
 			    }else{
 	
-			        if ( existing_file.exists()){
+			        if ( FileUtil.exists( existing_file )){
 	
 			            if ( !FileUtil.renameFile( existing_file, to_link, pl )){
 	
@@ -397,8 +398,6 @@ DiskManagerUtil
 
 	    state.setFileLink( file_info.getIndex(), from_file, to_link );
 
-	    state.save(false);
-
 	    return( null );
 	}
 
@@ -679,7 +678,7 @@ DiskManagerUtil
 									// files for dnd files which then force a recheck when the download
 									// starts for the first time)
 
-									if ( target_file.exists()){
+									if ( FileUtil.exists( target_file )){
 
 										CacheFile cache_file =
 											CacheFileManagerFactory.getSingleton().createFile(
@@ -703,7 +702,7 @@ DiskManagerUtil
 													public File
 													getCacheFileControlFileDir()
 													{
-														return( download_manager.getDownloadState().getStateFile( ));
+														return( download_manager.getDownloadState().getStateDir());
 													}
 													@Override
 													public int
@@ -711,8 +710,16 @@ DiskManagerUtil
 													{
 														return( CacheFileOwner.CACHE_MODE_NORMAL );
 													}
+													
+													@Override
+													public FileKey 
+													getCacheFileLink(
+														FileKey file )
+													{
+														return( download_manager.getDownloadState().getFileLink( idx, file ));
+													}
 												},
-												target_file,
+												new StringInterner.FileKey( target_file ),
 												DiskManagerUtil.convertDMStorageTypeToCache( newStorageType ), force );
 
 										try{
@@ -830,7 +837,7 @@ DiskManagerUtil
 
 		            	private volatile CacheFile   read_cache_file;
 		            	// do not access this field directly, use lazyGetFile() instead
-		            	private WeakReference dataFile = new WeakReference(null);
+		            	private WeakReference<StringInterner.FileKey> dataFile = new WeakReference<>(null);
 
 		            	private volatile Boolean skipping;
 		            	
@@ -985,144 +992,13 @@ DiskManagerUtil
 		            			// returns the file to link to if linkage is required
 
 		            		synchronized( DiskManagerUtil.skip_lock ){
+		            			
 			            		skipped_internal = _skipped;
 	
 		    					if ( !download_manager.isDestroyed()){
 	
-		    						DownloadManagerState dm_state = download_manager.getDownloadState();
-	
-				            		String dnd_sf = dm_state.getAttribute( DownloadManagerState.AT_DND_SUBFOLDER );
-	
-				            		if ( dnd_sf != null ){
-	
-				            			File	link = getLink();
-	
-				        				File 	file = getFile( false );
-	
-					            		if ( _skipped ){
-	
-					            			if ( link == null || link.equals( file )){
-	
-				            					File parent = file.getParentFile();
-	
-				            					if ( parent != null ){
-	
-		    										String prefix = dm_state.getAttribute( DownloadManagerState.AT_DND_PREFIX );
-	
-		    										String file_name = file.getName();
-	
-		    										if ( prefix != null && !file_name.startsWith( prefix )){
-	
-		    											file_name = prefix + file_name;
-		    										}
-	
-				            						File new_parent = FileUtil.newFile( parent, dnd_sf );
-	
-				            						File new_file = FileUtil.newFile( new_parent, file_name );
-	
-				            						if ( !new_file.exists()){
-	
-					            						if ( !new_parent.exists()){
-	
-					            							new_parent.mkdirs();
-					            						}
-	
-					            						if ( new_parent.canWrite()){
-	
-						            						boolean ok;
-	
-						            						if ( file.exists()){
-	
-						            							ok = FileUtil.renameFile( file, new_file );
-	
-						            						}else{
-	
-						            							ok = true;
-						            						}
-	
-						            						if ( ok ){
-	
-						            							return( new_file );
-						            						}
-					            						}
-				            						}
-				            					}
-					            			}
-					            		}else{
-	
-					            			if ( link != null && !file.exists()){
-	
-				            					File parent = file.getParentFile();
-	
-				            					if ( parent != null && parent.canWrite()){
-	
-				            						File new_parent = parent.getName().equals( dnd_sf )?parent:FileUtil.newFile( parent, dnd_sf );
-	
-				            							// use link name to handle incomplete file suffix if set
-	
-				            						File new_file = FileUtil.newFile( new_parent, link.getName());
-	
-				            						if ( new_file.equals( link )){
-	
-				            							boolean	ok;
-	
-				            							String incomp_ext = dm_state.getAttribute( DownloadManagerState.AT_INCOMP_FILE_SUFFIX );
-	
-				    									String file_name = file.getName();
-	
-				    									String prefix = dm_state.getAttribute( DownloadManagerState.AT_DND_PREFIX );
-	
-				    									boolean prefix_removed = false;
-	
-				    									if ( prefix != null && file_name.startsWith(prefix)){
-	
-				    										file_name = file_name.substring( prefix.length());
-	
-				    										prefix_removed = true;
-				    									}
-	
-				    									if  ( 	incomp_ext != null && incomp_ext.length() > 0 &&
-				    											getDownloaded() != getLength()){
-	
-																// retain the prefix if enabled and we have a suffix
-	
-				    										if ( prefix == null ){
-	
-				    											prefix = "";
-				    										}
-	
-				    										file = FileUtil.newFile( file.getParentFile(), prefix + file_name + incomp_ext );
-	
-				    									}else if ( prefix_removed ){
-	
-				    										file = FileUtil.newFile( file.getParentFile(), file_name);
-				    									}
-	
-				            							if ( new_file.exists()){
-	
-				            								ok = FileUtil.renameFile( new_file, file );
-	
-				            							}else{
-	
-				            								ok = true;
-				            							}
-	
-				            							if ( ok ){
-	
-				            								File[] files = new_parent.listFiles();
-	
-				            								if ( files != null && files.length == 0 ){
-	
-				            									new_parent.delete();
-				            								}
-	
-				            								return( file );
-				            							}
-				            						}
-				            					}
-					            			}
-					            		}
-				            		}
+		    						return( setSkippedInternalSupport( download_manager, this, null, _skipped ));
+				            		
 		    					}
 	
 		    					return( null );
@@ -1178,48 +1054,29 @@ DiskManagerUtil
 
 		            	private File lazyGetFile()
 		            	{
-		            		File toReturn = (File)dataFile.get();
-		            		if(toReturn != null)
-		            			return toReturn;
-
-		            		TOTorrent tor = download_manager.getTorrent();
-
-		            		File  dataPath = root_dir;
-		            		File simpleFile = null;
-
-		            		// for a simple torrent the target file can be changed
-
-		            		if ( tor == null || tor.isSimpleTorrent()){
-
-		            				// rumour has it tor can sometimes be null
-
-		            			simpleFile = download_manager.getAbsoluteSaveLocation();
-
-		            		}else{
-		            			byte[][]path_comps = torrent_file.getPathComponents();
-
-		            			for (int j=0;j<path_comps.length;j++){
-
-		            				String comp;
-		            				try
-		            				{
-		            					comp = locale_decoder.decodeString( path_comps[j] );
-		            				} catch (UnsupportedEncodingException e)
-		            				{
-		            					Debug.printStackTrace(e);
-		            					comp = "undecodableFileName"+file_index;
-		            				}
-
-		            				comp = FileUtil.convertOSSpecificChars( comp,  j != path_comps.length-1 );
-
-		            				dataPath = FileUtil.newFile(dataPath, comp);
-		            			}
+		            		StringInterner.FileKey fk = dataFile.get();
+		            		
+		            		if ( fk == null ){
+		            					            		
+			            		TOTorrent tor = download_manager.getTorrent();
+	
+			            		// for a simple torrent the target file can be changed
+	
+			            		if ( tor == null || tor.isSimpleTorrent()){
+	
+			            				// rumour has it tor can sometimes be null
+	
+			            			fk = new StringInterner.FileKey( download_manager.getAbsoluteSaveLocation());
+	
+			            		}else{
+	
+			            			fk = new StringInterner.FileKey( root_dir, torrent_file.getRelativePath( locale_decoder ));
+			            		}
+	
+			            		dataFile = new WeakReference<>( fk );
 		            		}
-
-		            		dataFile = new WeakReference(toReturn = simpleFile != null ? simpleFile : dataPath );
-
-		            		//System.out.println("new file:"+toReturn);
-		            		return toReturn;
+		            		
+	            			return( fk.getFile());
 		            	}
 
 		            	@Override
@@ -1290,7 +1147,7 @@ DiskManagerUtil
 		                public File
 	                	getLink()
 	                	{
-	                		return( download_manager.getDownloadState().getFileLink( file_index, lazyGetFile() ));
+	                		return( download_manager.getDownloadState().getFileLink( file_index));
 	                	}
 
 	                	@Override
@@ -1353,7 +1210,7 @@ DiskManagerUtil
 									                public File
 	                								getCacheFileControlFileDir()
 	                								{
-	                									return( download_manager.getDownloadState().getStateFile( ));
+	                									return( download_manager.getDownloadState().getStateDir());
 	                								}
 	                								@Override
 									                public int
@@ -1361,8 +1218,15 @@ DiskManagerUtil
 	                								{
 	                									return( CacheFileOwner.CACHE_MODE_NORMAL );
 	                								}
+	                								@Override
+													public FileKey 
+													getCacheFileLink(
+														FileKey file )
+													{
+														return( download_manager.getDownloadState().getFileLink( file_index, file ));
+													}
 	                							},
-	                							getFile( true ),
+	                							new StringInterner.FileKey( getFile( true )),
 	                							convertDMStorageTypeToCache( type ), false );
 
 	                				}catch( Throwable e ){
@@ -1786,7 +1650,8 @@ DiskManagerUtil
 	
 	public static DiskManagerPiece[]
 	getDiskManagerPiecesSnapshot(
-		DownloadManager		dm )
+		DownloadManager				dm,
+		DiskManagerFileInfoSet		file_set )
 	{
 		try{
 			TOTorrent torrent = dm.getTorrent();
@@ -1805,6 +1670,7 @@ DiskManagerUtil
 	
 	        DiskManagerPiece[] pieces      = new DiskManagerPiece[nbPieces];
 
+	        /*
 			DiskManagerFileInfoSet file_set = 
 				getFileInfoSkeleton(
 					dm,
@@ -1822,7 +1688,8 @@ DiskManagerUtil
 						public void filePriorityChanged(DiskManager dm, List<DiskManagerFileInfo> files){
 						}
 					});
-				
+			*/
+	        
 			DiskManagerFileInfo[] files = file_set.getFiles();
 	
 	        LocaleUtilDecoder   locale_decoder = LocaleTorrentUtil.getTorrentEncoding( torrent );
@@ -2307,7 +2174,8 @@ DiskManagerUtil
 			FileUtil.runAsTask(
 				new CoreOperationTask()
 				{
-					private boolean		queued = true;
+					private boolean		added_to_tasks	= false;
+					private boolean		queued			= true;
 
 					@Override
 					public String 
@@ -2331,91 +2199,91 @@ DiskManagerUtil
 					}
 
 					private ProgressCallback callback = 
-					new ProgressCallbackAdapter( move_smallest_first?ProgressCallbackAdapter.SORT_SIZE:ProgressCallbackAdapter.SORT_ALLOC_ORDER )
-					{
-						private volatile int	current_state = ProgressCallback.ST_NONE;
-
-						@Override
-						public int 
-						getProgress()
+						new ProgressCallbackAdapter( move_smallest_first?ProgressCallbackAdapter.SORT_SIZE:ProgressCallbackAdapter.SORT_ALLOC_ORDER )
 						{
-							long[] mp = adapter.getMoveProgress();
-
-							return( mp==null?0:(int)mp[0]);
-						}
-
-						@Override
-						public long 
-						getSize()
-						{
-							long[] mp = adapter.getMoveProgress();
-
-							return( mp==null?download_manager.getStats().getSizeExcludingDND():mp[1] );
-						}
-
-						@Override
-						public String 
-						getSubTaskName()
-						{
-							return( adapter.getMoveSubTask());
-						}
-
-						@Override
-						public int 
-						getSupportedTaskStates()
-						{
-							return( 
-									ProgressCallback.ST_PAUSE |  
-									ProgressCallback.ST_RESUME |
-									ProgressCallback.ST_CANCEL |
-									ProgressCallback.ST_SUBTASKS );
-						}
-
-						@Override
-						public void 
-						setTaskState(
-								int state )
-						{
-							if ( current_state == ProgressCallback.ST_CANCEL ){
-
-								return;
+							private volatile int	current_state = ProgressCallback.ST_NONE;
+	
+							@Override
+							public int 
+							getProgress()
+							{
+								long[] mp = adapter.getMoveProgress();
+	
+								return( mp==null?0:(int)mp[0]);
 							}
-
-							if ( state == ProgressCallback.ST_PAUSE ){
-
-								adapter.setMoveState( ProgressListener.ST_PAUSED );
-
-								current_state = state;
-
-							}else if ( state == ProgressCallback.ST_RESUME ){
-
-								adapter.setMoveState( ProgressListener.ST_NORMAL );
-
-								current_state = ProgressCallback.ST_NONE;									
-
-							}else if ( state == ProgressCallback.ST_CANCEL ){
-
-								adapter.setMoveState( ProgressListener.ST_CANCELLED );
-
-								current_state = ProgressCallback.ST_CANCEL;									
+	
+							@Override
+							public long 
+							getSize()
+							{
+								long[] mp = adapter.getMoveProgress();
+	
+								return( mp==null?download_manager.getStats().getSizeExcludingDND():mp[1] );
 							}
-						}
-
-						public int
-						getTaskState()
-						{
-							if ( current_state == ProgressCallback.ST_NONE && queued ){
-
-								return( ProgressCallback.ST_QUEUED );
+	
+							@Override
+							public String 
+							getSubTaskName()
+							{
+								return( adapter.getMoveSubTask());
 							}
-
-							return( current_state );
-						}
-					};
-
+	
+							@Override
+							public int 
+							getSupportedTaskStates()
+							{
+								return( 
+										ProgressCallback.ST_PAUSE |  
+										ProgressCallback.ST_RESUME |
+										ProgressCallback.ST_CANCEL |
+										ProgressCallback.ST_SUBTASKS );
+							}
+	
+							@Override
+							public void 
+							setTaskState(
+									int state )
+							{
+								if ( current_state == ProgressCallback.ST_CANCEL ){
+	
+									return;
+								}
+	
+								if ( state == ProgressCallback.ST_PAUSE ){
+	
+									adapter.setMoveState( ProgressListener.ST_PAUSED );
+	
+									current_state = state;
+	
+								}else if ( state == ProgressCallback.ST_RESUME ){
+	
+									adapter.setMoveState( ProgressListener.ST_NORMAL );
+	
+									current_state = ProgressCallback.ST_NONE;									
+	
+								}else if ( state == ProgressCallback.ST_CANCEL ){
+	
+									adapter.setMoveState( ProgressListener.ST_CANCELLED );
+	
+									current_state = ProgressCallback.ST_CANCEL;									
+								}
+							}
+	
+							public int
+							getTaskState()
+							{
+								if ( current_state == ProgressCallback.ST_NONE && queued ){
+	
+									return( ProgressCallback.ST_QUEUED );
+								}
+	
+								return( current_state );
+							}
+						};
+	
 					@Override
-					public void
-					run(
+					public boolean
+					runOperation(
 						CoreOperation operation)
 					{
 						boolean has_scheduler = DiskManagerOperationScheduler.isEnabled();
@@ -2447,6 +2315,11 @@ DiskManagerUtil
 
 											throw( new RuntimeException( "Cancelled" ));
 										}
+										
+											// come back later
+										
+										return( false );
+										
 									}else{
 										
 										break;
@@ -2462,7 +2335,12 @@ DiskManagerUtil
 
 						synchronized( move_tasks ){
 
-							move_tasks.add( this );
+							if ( !added_to_tasks ){
+								
+								move_tasks.add( this );
+								
+								added_to_tasks = true;
+							}
 
 							ready = move_tasks.size() == 1;
 						}
@@ -2478,6 +2356,10 @@ DiskManagerUtil
 									}catch( Throwable e ){
 
 									}
+									
+										// come back later
+									
+									return( false );
 								}
 
 								if ( callback.getTaskState() == ProgressCallback.ST_CANCEL ){
@@ -2521,6 +2403,13 @@ DiskManagerUtil
 											}
 										}
 									}
+									
+									if ( !ready ){
+										
+											// come back later
+										
+										return( false );
+									}
 								}
 							}
 
@@ -2528,6 +2417,8 @@ DiskManagerUtil
 
 							target.run();
 
+							return( true );
+							
 						}finally{
 
 							synchronized( move_tasks ){
@@ -2536,7 +2427,7 @@ DiskManagerUtil
 							}
 						}
 					}
-
+				
 					@Override
 					public ProgressCallback 
 					getProgressCallback()
@@ -2571,4 +2462,420 @@ DiskManagerUtil
 		setMoveState(
 			int	state );
 	}
+	
+	
+	protected static File
+	setSkippedInternalSupport(
+		DownloadManager			dm,
+		DiskManagerFileInfo		dm_file,
+		CacheFile				cache_file,
+		boolean					skipped )
+	{
+		DownloadManagerState	dm_state = dm.getDownloadState();
+		
+		int file_index = dm_file.getIndex();
+				
+		String dnd_sf = dm_state.getAttribute( DownloadManagerState.AT_DND_SUBFOLDER );
+
+ 		if ( dnd_sf != null ){
+
+			File	link = dm_file.getLink();				// where the file currently is, null if not amended at all
+
+			File 	base_file = dm_file.getFile( false );	// where the file should be without any renaming/moving
+
+    		if ( skipped ){
+
+				File	base_parent = base_file.getParentFile();
+				
+				boolean file_ok = base_parent != null;
+				
+				if ( file_ok ){	
+					
+	    				// if the file isn't pretty much where we expect it to be then we don't touch it
+	    			
+	    			file_ok = link == null || link.equals( base_file );
+	    			
+	    			if ( !file_ok ){
+	    				
+	    					// not identical but might still be good enough
+	    				
+	    				if ( base_parent != null && base_parent.equals( link.getParentFile())){
+	    					
+	    						// that'll do, we don't really care about the name to move it
+	    					
+	    					file_ok = true;
+	    				}
+	    			}
+				}
+				
+    			if ( file_ok ){
+
+					String new_name = link==null?base_file.getName():link.getName();
+
+						// just in case add in any expected stuff
+					
+					String prefix = dm_state.getAttribute( DownloadManagerState.AT_DND_PREFIX );
+
+					String incomp_ext = dm_state.getAttribute( DownloadManagerState.AT_INCOMP_FILE_SUFFIX );
+
+					boolean incomplete = dm_file.getDownloaded() != dm_file.getLength();
+					
+					if ( prefix != null && incomplete && !new_name.startsWith( prefix )){
+
+						new_name = prefix + new_name;
+					}
+
+					if ( incomp_ext != null && incomplete && !new_name.endsWith( incomp_ext )){
+												
+						new_name = new_name + incomp_ext;
+					}
+					
+					File new_parent = FileUtil.newFile( base_parent, dnd_sf );
+
+					File new_file = FileUtil.newFile( new_parent, new_name );
+
+					if ( !FileUtil.exists( new_file )){
+
+						if ( !FileUtil.exists( new_parent )){
+
+							new_parent.mkdirs();
+						}
+
+						if ( new_parent.canWrite()){
+
+							if ( cache_file == null ){
+								
+								boolean ok;
+								
+		   						File source = link==null?base_file:link;
+	    						
+	    						if ( FileUtil.exists( source )){
+
+	    							ok = FileUtil.renameFile( source, new_file );
+
+	    						}else{
+
+	    								// might not be allocated yet
+	    							
+	    							ok = true;
+	    							
+	    							File[] files = new_parent.listFiles();
+									
+									if ( files != null && files.length == 0 ){
+
+										new_parent.delete();
+									}
+	    						}
+
+	    						if ( ok ){
+
+	    							return( new_file );
+	    						}
+							}else{
+	    						boolean ok;
+	
+								try{
+									dm_state.setFileLink( file_index, base_file, new_file );
+	
+									cache_file.moveFile( new_file, null );
+	
+									ok = true;
+	
+								}catch( Throwable e ){
+	
+									ok = false;
+	
+									Debug.out( e );
+								}
+	
+	    						if ( !ok ){
+	
+	    							dm_state.setFileLink( file_index, base_file, link );
+	    						}
+							}
+						}
+					}
+    			}
+    		}else{
+
+ 					// here we are removing the added folder so there must be a link
+    			
+    			if ( link != null ){
+
+					File link_parent = link.getParentFile();
+
+					if ( link_parent != null && link_parent.getName().equals( dnd_sf )){
+
+						File new_parent = link_parent.getParentFile();
+
+						if ( new_parent != null ){
+									
+							File new_file = FileUtil.newFile( new_parent, link.getName());
+							
+							if ( cache_file == null ){
+								
+									// inactive case
+								
+								File result = null;
+								
+								if ( FileUtil.exists( link )){
+									
+									if ( FileUtil.renameFile( link, new_file )){
+		
+										result = new_file;
+									}
+								}else{
+									
+										// file might not be allocated yet
+									
+									result = new_file;
+								}
+								
+								if ( FileUtil.exists( link_parent )){
+									
+									File[] files = link_parent.listFiles();
+									
+									if ( files != null && files.length == 0 ){
+
+										link_parent.delete();
+									}
+								}
+								
+								return( result );
+								
+							}else{
+					
+									// active case
+								
+								boolean	ok;
+	
+								try{
+	
+									dm_state.setFileLink( file_index, base_file, new_file );
+	
+									cache_file.moveFile( new_file, null );
+									
+									File[] files = link_parent.listFiles();
+	
+									if ( files != null && files.length == 0 ){
+	
+										link_parent.delete();
+									}
+	
+									ok = true;
+	
+								}catch( Throwable e ){
+	
+									ok = false;
+	
+									Debug.out( e );
+								}
+	
+								if ( !ok ){
+	
+									dm_state.setFileLink( file_index, base_file, link );
+	    						}
+							}
+						}
+					}
+    			}
+    		}
+    		
+    		return( null );
+		}
+	
+		String dnd_al = dm_state.getAttribute( DownloadManagerState.AT_DND_ALT_LOC );
+
+ 		if ( dnd_al != null ){
+
+ 			File dnd_root	= FileUtil.newFile( dnd_al );
+ 			 			
+ 			String save_location_path = dm.getSaveLocation().getAbsolutePath();
+ 			
+			File	link = dm_file.getLink();				// where the file currently is, null if not amended at all
+
+			File 	base_file = dm_file.getFile( false );	// where the file should be without any renaming/moving
+
+    		if ( skipped ){ 
+   			
+				File	base_parent = base_file.getParentFile();
+				
+				boolean file_ok = base_parent != null && base_file.getAbsolutePath().startsWith( save_location_path );
+				
+				if ( file_ok ){	
+					
+	    				// if the file isn't pretty much where we expect it to be then we don't touch it
+	    			
+	    			file_ok = link == null || link.equals( base_file );
+	    			
+	    			if ( !file_ok ){
+	    				
+	    					// not identical but might still be good enough
+	    				
+	    				if ( base_parent != null && base_parent.equals( link.getParentFile())){
+	    					
+	    						// that'll do, we don't really care about the name to move it
+	    					
+	    					file_ok = true;
+	    				}
+	    			}
+				}
+				
+    			if ( file_ok ){
+
+					String new_name = link==null?base_file.getName():link.getName();
+
+						// just in case add in any expected stuff
+					
+					String prefix = dm_state.getAttribute( DownloadManagerState.AT_DND_PREFIX );
+
+					String incomp_ext = dm_state.getAttribute( DownloadManagerState.AT_INCOMP_FILE_SUFFIX );
+
+					boolean incomplete = dm_file.getDownloaded() != dm_file.getLength();
+					
+					if ( prefix != null && incomplete && !new_name.startsWith( prefix )){
+
+						new_name = prefix + new_name;
+					}
+
+					if ( incomp_ext != null && incomplete && !new_name.endsWith( incomp_ext )){
+												
+						new_name = new_name + incomp_ext;
+					}
+					
+					File new_parent = FileUtil.newFile( dnd_root, base_parent.getAbsolutePath().substring( save_location_path.length()));
+
+					File new_file = FileUtil.newFile( new_parent, new_name );
+
+					if ( !FileUtil.exists( new_file )){
+
+						if ( !FileUtil.exists( new_parent )){
+
+							new_parent.mkdirs();
+						}
+
+						if ( new_parent.canWrite()){
+
+							if ( cache_file == null ){
+								
+								boolean ok;
+								
+		   						File source = link==null?base_file:link;
+	    						
+	    						if ( FileUtil.exists( source )){
+
+	    							ok = FileUtil.renameFile( source, new_file );
+
+	    						}else{
+
+	    								// might not be allocated yet
+	    							
+	    							ok = true;
+	    							
+	    							File[] files = new_parent.listFiles();
+									
+									if ( files != null && files.length == 0 ){
+
+										new_parent.delete();
+									}
+	    						}
+
+	    						if ( ok ){
+
+	    							return( new_file );
+	    						}
+							}else{
+	    						boolean ok;
+	
+								try{
+									dm_state.setFileLink( file_index, base_file, new_file );
+	
+									cache_file.moveFile( new_file, null );
+	
+									ok = true;
+	
+								}catch( Throwable e ){
+	
+									ok = false;
+	
+									Debug.out( e );
+								}
+	
+	    						if ( !ok ){
+	
+	    							dm_state.setFileLink( file_index, base_file, link );
+	    						}
+							}
+						}
+					}
+    			}
+    		}else{
+
+ 					// here we are removing the added folder so there must be a link
+    			
+    			if ( link != null ){
+
+					if ( link.getAbsolutePath().startsWith( dnd_root.getAbsolutePath())){
+
+						File new_parent = base_file.getParentFile();
+
+						if ( new_parent != null ){
+									
+							File new_file = FileUtil.newFile( new_parent, link.getName());
+							
+							if ( cache_file == null ){
+								
+									// inactive case
+								
+								File result = null;
+								
+								if ( FileUtil.exists( link )){
+									
+									if ( FileUtil.renameFile( link, new_file )){
+		
+										result = new_file;
+									}
+								}else{
+									
+										// file might not be allocated yet
+									
+									result = new_file;
+								}
+								
+								return( result );
+								
+							}else{
+					
+									// active case
+								
+								boolean	ok;
+	
+								try{
+	
+									dm_state.setFileLink( file_index, base_file, new_file );
+	
+									cache_file.moveFile( new_file, null );
+	
+									ok = true;
+	
+								}catch( Throwable e ){
+	
+									ok = false;
+	
+									Debug.out( e );
+								}
+	
+								if ( !ok ){
+	
+									dm_state.setFileLink( file_index, base_file, link );
+	    						}
+							}
+						}
+					}
+    			}
+    		}
+		}
+ 		
+ 		return( null );
+	}
 }
