@@ -119,6 +119,7 @@ public class SideBar
 	private DropTarget dropTarget;
 
 	protected SideBarEntrySWT draggingOver;
+	protected SideBarEntrySWT mousingOver;
 
 	private Color fg;
 
@@ -136,6 +137,9 @@ public class SideBar
 	private ParameterListener configBGColorListener;
 	private SWTViewListener swtViewListener;
 
+	private SideBarEntrySWT	activeMenuEntry;
+	private SideBarEntrySWT	lastMenuEntry;
+	
 	public SideBar() {
 		super(null, UISWTInstance.VIEW_MAIN, null);
 		setCloseableConfigFile("sidebarauto.config");
@@ -210,7 +214,7 @@ public class SideBar
 					event.character = '\0';
 					flipSideBarVisibility();
 				}else if (event.keyCode == SWT.F4 && event.stateMask == SWT.CTRL ){
-					MdiEntry entry = getCurrentEntry();
+					MdiEntry entry = getSelectedEntry();
 
 					if ( entry instanceof SideBarEntrySWT && entry.isCloseable()){
 
@@ -270,7 +274,7 @@ public class SideBar
 	
 					@Override
 					public void menuWillBeShown(MenuItem menu, Object data) {
-						SideBarEntrySWT sbe = getCurrentEntry();
+						SideBarEntrySWT sbe = getMenuEntry( false );
 	
 						if ( sbe != null && !sbe.isCloseable()){
 							
@@ -285,7 +289,7 @@ public class SideBar
 					}
 				});
 	
-			menuItem.addListener((menu, target) -> closeEntry(getCurrentEntry(), true ));
+			menuItem.addListener((menu, target) -> closeEntry(getMenuEntry( false ), true ));
 		}
 		
 		{
@@ -298,7 +302,7 @@ public class SideBar
 	
 					@Override
 					public void menuWillBeShown(MenuItem menu, Object data) {
-						SideBarEntrySWT sbe = getCurrentEntry();
+						SideBarEntrySWT sbe = getMenuEntry( false );
 	
 						if ( sbe != null ){
 							String id = sbe.getId();
@@ -315,7 +319,7 @@ public class SideBar
 				});
 	
 			menuItem.addListener((menu, target) -> {
-				SideBarEntrySWT sbe = getCurrentEntry();
+				SideBarEntrySWT sbe = getMenuEntry( false );
 				
 				if ( sbe != null ){
 					
@@ -334,7 +338,7 @@ public class SideBar
 			
 			menuParentItem.addFillListener((menu, data) -> {
 				
-				SideBarEntrySWT entry = getCurrentEntry();
+				SideBarEntrySWT entry = getMenuEntry( false );
 				
 				boolean visible = 
 					entry != null && 
@@ -366,7 +370,7 @@ public class SideBar
 			menuItemDashBoard.addListener(new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 						
@@ -380,7 +384,7 @@ public class SideBar
 			menuItemTopbar.addListener(new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 						
@@ -394,7 +398,7 @@ public class SideBar
 			menuItemSidebar.addListener(new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 						
@@ -408,7 +412,7 @@ public class SideBar
 			menuItemRightbar.addListener(new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 						
@@ -422,7 +426,7 @@ public class SideBar
 			menuItemQuickLinks.addListener(new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 						
@@ -443,7 +447,7 @@ public class SideBar
 	
 					@Override
 					public void menuWillBeShown(MenuItem menu, Object data) {
-						SideBarEntrySWT sbe = getCurrentEntry();
+						SideBarEntrySWT sbe = getMenuEntry( false );
 	
 						menu.setVisible( sbe != null && sbe.canBuildStandAlone());
 					}
@@ -455,7 +459,7 @@ public class SideBar
 			MenuItemListener listener = new MenuItemListener() {
 				@Override
 				public void selected(MenuItem menu, Object target) {
-					SideBarEntrySWT sbe = getCurrentEntry();
+					SideBarEntrySWT sbe = getMenuEntry( false );
 	
 					if ( sbe != null ){
 
@@ -778,7 +782,8 @@ public class SideBar
 			TreeItem lastTopItem = null;
 
 			boolean mouseDowned = false;
-
+			int		mouseDownedButton;
+			
 			Rectangle lastCloseAreaClicked = null;
 
 			private boolean wasExpanded;
@@ -817,7 +822,7 @@ public class SideBar
 							SideBarEntrySWT entry = (SideBarEntrySWT) treeItem.getData("MdiEntry");
 							//System.out.println("PaintItem: " + event.item + ";" + event.index + ";" + event.detail + ";" + event.getBounds() + ";" + event.gc.getClipping());
 							if (entry != null) {
-								boolean selected = getCurrentEntry() == entry;
+								boolean selected = getSelectedEntry() == entry;
 									
 								if (!selected) {
 									event.detail &= ~SWT.SELECTED;
@@ -849,7 +854,7 @@ public class SideBar
 								if (itemBounds != null && entry != null) {
 									event.item = treeItem;
 
-									boolean selected = getCurrentEntry() == entry
+									boolean selected = getSelectedEntry() == entry
 											&& entry.isSelectable();
 									event.detail = selected ? SWT.SELECTED : SWT.NONE;
 
@@ -907,21 +912,48 @@ public class SideBar
 								return;
 							}
 							SideBarEntrySWT entry = (SideBarEntrySWT) treeItem.getData("MdiEntry");
+							
+							if ( Constants.isOSX ){
+									// need this as moving right-click menu from one
+									// entry to another doesn't result in any mouse move events
+								
+								if ( mousingOver != entry ){
+									SideBarEntrySWT old = mousingOver;
+									
+									mousingOver = entry;
+									if ( old != null ){
+										old.redraw();
+									}
+									if ( entry != null ){
+										
+										entry.redraw();
+									}
+								}
+							}
+							
 							if (entry != null ) {
 								Point cursorLocation = tree.toControl(event.display.getCursorLocation());
 								if (lastCloseAreaClicked != null && lastCloseAreaClicked.contains(cursorLocation.x, cursorLocation.y)) {
 									return;
 								}
 
-								showEntry(entry);
-							} else if (getCurrentEntry() != null) {
+								if ( mouseDowned && mouseDownedButton == 3 ){
+									
+										// if you want right-click sidebar menu to force selection of the entry before
+										// showing the menu then uncomment this
+									
+									// showEntry(entry);
+								}else{
+									showEntry(entry);
+								}
+							} else if (getSelectedEntry() != null) {
 								TreeItem topItem = tree.getTopItem();
 
 								// prevent "jumping" in the case where selection is off screen
 								// setSelection would jump the item on screen, and then
 								// showItem would jump back to where the user was.
 								tree.setRedraw(false);
-								TreeItem ti = getCurrentEntry().getTreeItem();
+								TreeItem ti = getSelectedEntry().getTreeItem();
 								if (ti != null) {
 									tree.setSelection(ti);
 								}
@@ -936,13 +968,35 @@ public class SideBar
 							}
 							break;
 						}
-
+						case SWT.MouseExit: {
+							SideBarEntrySWT old = mousingOver;
+							mousingOver = null;
+							if ( old != null ){
+								old.redraw();
+							}
+							
+							break;
+						}
+						case SWT.MouseEnter:
 						case SWT.MouseMove: {
 							int indent = END_INDENT ? tree.getClientArea().width - 1 : 0;
 							treeItem = tree.getItem(new Point(indent, event.y));
 							SideBarEntrySWT entry = (SideBarEntrySWT) ((treeItem == null || treeItem.isDisposed())
 									? null : treeItem.getData("MdiEntry"));
 
+							if ( mousingOver != entry ){
+								SideBarEntrySWT old = mousingOver;
+								
+								mousingOver = entry;
+								if ( old != null ){
+									old.redraw();
+								}
+								if ( entry != null ){
+									
+									entry.redraw();
+								}
+							}
+							
 							int cursorNo = SWT.CURSOR_ARROW;
 							if (treeItem != null) {
 								Rectangle closeArea = (Rectangle) treeItem.getData("closeArea");
@@ -988,6 +1042,8 @@ public class SideBar
 
 						case SWT.MouseDown: {
 							mouseDowned = true;
+							mouseDownedButton = event.button;
+							
 							lastCloseAreaClicked  = null;
 							if (tree.getItemCount() == 0 || event.button != 1) {
 								return;
@@ -1031,7 +1087,7 @@ public class SideBar
 							if (closeArea != null && closeArea.contains(event.x, event.y)) {
 								//treeItem.dispose();
 								return;
-							} else if (getCurrentEntry() != entry && Constants.isOSX) {
+							} else if (getSelectedEntry() != entry && Constants.isOSX) {
 								// showEntry(entry);  removed as we'll get a selection event if needed
 							}
 
@@ -1065,7 +1121,7 @@ public class SideBar
 										
 											// Note: On Windows, user can expand row by clicking the invisible area where the OS twisty would be
 										
-										MdiEntry currentEntry = getCurrentEntry();
+										MdiEntry currentEntry = getSelectedEntry();
 										
 										String collapsedID = entry.getViewID();
 										
@@ -1098,7 +1154,7 @@ public class SideBar
 							SideBarEntrySWT entry = (SideBarEntrySWT) ((treeItem == null || treeItem.isDisposed())
 									? null : treeItem.getData("MdiEntry"));
 
-							if ( entry != null ){
+							if ( entry != null && ( event.button == 1 || event.button == 2 )){
 								
 									// don't trigger popout if double click is in close area or
 									// item has child and user is clicking in the expandy area 
@@ -1145,7 +1201,7 @@ public class SideBar
 						case SWT.Collapse: {
 							SideBarEntrySWT entry = (SideBarEntrySWT) treeItem.getData("MdiEntry");
 							
-							MdiEntry currentEntry = getCurrentEntry();
+							MdiEntry currentEntry = getSelectedEntry();
 							
 							String collapsedID = entry.getViewID();
 							
@@ -1167,7 +1223,7 @@ public class SideBar
 						}
 						case SWT.KeyDown:{
 						
-							MdiEntry currentEntry = getCurrentEntry();
+							MdiEntry currentEntry = getSelectedEntry();
 							
 							if ( currentEntry != null ){
 								
@@ -1200,7 +1256,10 @@ public class SideBar
 		tree.addListener(SWT.MouseDown, treeListener);
 
 		// For cursor
+		tree.addListener(SWT.MouseEnter, treeListener);
 		tree.addListener(SWT.MouseMove, treeListener);
+		// move hover
+		tree.addListener(SWT.MouseExit, treeListener);
 
 		// to disable collapsing
 		tree.addListener(SWT.Collapse, treeListener);
@@ -1328,6 +1387,15 @@ public class SideBar
 			public void menuHidden(MenuEvent e) {
 				bShown = false;
 
+				SideBarEntrySWT menuEntry = getMenuEntry( false );
+					
+				if ( menuEntry != null ){
+											
+					setMenuEntry( null );
+					
+					menuEntry.redraw();
+				}
+				
 				if (Constants.isOSX) {
 					return;
 				}
@@ -1350,15 +1418,15 @@ public class SideBar
 				Utils.disposeSWTObjects(menuTree.getItems());
 
 				bShown = true;
-
-				SideBarEntrySWT entry = (SideBarEntrySWT) tree.getData("MdiEntry");
 				
-				if (entry == null) {
-					entry = getCurrentEntry(); 
-				}
+				SideBarEntrySWT entry = (SideBarEntrySWT)menuTree.getData("MdiEntry");
 
+				setMenuEntry( entry );
+				
 				if ( entry != null ){
 				
+					entry.redraw();
+					
 					fillMenu(menuTree, entry, "sidebar");
 				}
 
@@ -1476,7 +1544,7 @@ public class SideBar
 					@Override
 					public void pressed(SWTSkinButtonUtility buttonUtility,
 					                    SWTSkinObject skinObject, int stateMask) {
-						closeEntry(getCurrentEntry(),true);
+						closeEntry( getSelectedEntry(),true);
 					}
 				});
 			}
@@ -1650,8 +1718,14 @@ public class SideBar
 	 *
 	 * @since 3.1.1.1
 	 */
-	protected void fillDropDownMenu(Menu menuDropDown, TreeItem[] items,
-			int indent) {
+	protected boolean 
+	fillDropDownMenu(
+		Menu		menuDropDown, 
+		TreeItem[]	items,
+		int 		indent) 
+	{
+		boolean hit = false;
+
 		String s = "";
 		for (int i = 0; i < indent; i++) {
 			s += "   ";
@@ -1677,11 +1751,16 @@ public class SideBar
 					//ind = "\t" + o;
 				}
 			}
-			menuItem.setText(s + entry.getTitle() + ind);
+			String title = s + entry.getTitle() + ind;
+			
+			menuItem.setText( title );
 			menuItem.addSelectionListener(dropDownSelectionListener);
-			MdiEntry currentEntry = getCurrentEntry();
+			MdiEntry currentEntry = getSelectedEntry();
+						
 			if (currentEntry != null && currentEntry.getViewID().equals(id)) {
 				menuItem.setSelection(true);
+				
+				hit = true;
 			}
 
 			TreeItem[] subItems = treeItem.getItems();
@@ -1693,9 +1772,19 @@ public class SideBar
 				}
 
 
-				fillDropDownMenu(parent, subItems, indent + 1);
+				if ( fillDropDownMenu(parent, subItems, indent + 1)){
+					
+					hit = true;
+					
+					if ( !entry.isSelectable()){
+						
+						menuItem.setText( title + " (*)");
+					}
+				};
 			}
 		}
+		
+		return( hit );
 	}
 
 	/**
@@ -2053,14 +2142,14 @@ public class SideBar
 	}
 
 	@Override
-	protected void setCurrentEntry(MdiEntrySWT entry){
+	protected void setSelectedEntry(MdiEntrySWT entry){
 		
 		if (entry != null && btnCloseItem != null ){
 			
 			btnCloseItem.setDisabled( !entry.isCloseable());
 		}
 		
-		super.setCurrentEntry(entry);
+		super.setSelectedEntry(entry);
 	}
 	
 	@Override
@@ -2073,7 +2162,7 @@ public class SideBar
 			return;
 		}
 
-		final SideBarEntrySWT oldEntry = getCurrentEntry();
+		final SideBarEntrySWT oldEntry = getSelectedEntry();
 
 		//System.out.println("showEntry " + newEntry.getId() + "; was " + (oldEntry == null ? "null" : oldEntry.getId()) + " via " + Debug.getCompressedStackTrace());
 		if (oldEntry == newEntry) {
@@ -2082,7 +2171,7 @@ public class SideBar
 		}
 
 		// show new
-		setCurrentEntry((MdiEntrySWT)newEntry );
+		setSelectedEntry((MdiEntrySWT)newEntry );
 
 		if (oldEntry != null && oldEntry != newEntry) {
 			oldEntry.redraw();
@@ -2107,7 +2196,7 @@ public class SideBar
 			if (entry == null) {
 				continue;
 			}
-			if ( entry != getCurrentEntry()){
+			if ( entry != getSelectedEntry()){
 
 				SWTSkinObject obj = ((SideBarEntrySWT)entry).getSkinObjectMaster();
 
@@ -2386,7 +2475,7 @@ public class SideBar
 
 	@Override
 	public BaseMdiEntry closeEntryByID(String id, boolean user_initiated ) {
-		MdiEntry currentBeforeClose = getCurrentEntry();
+		MdiEntry currentBeforeClose = getSelectedEntry();
 
 		BaseMdiEntry entry = super.closeEntryByID(id, user_initiated);
 		if (entry == null || Utils.isDisplayDisposed()) {
@@ -2454,7 +2543,7 @@ public class SideBar
 	private void
 	showNext()
 	{
-		MdiEntry current = getCurrentEntry();
+		MdiEntry current = getSelectedEntry();
 
 		String next = null;
 		
@@ -2489,7 +2578,7 @@ public class SideBar
 	private void
 	showPrevious()
 	{
-		MdiEntry current = getCurrentEntry();
+		MdiEntry current = getSelectedEntry();
 		
 		String next = null;
 
@@ -2593,8 +2682,28 @@ public class SideBar
 	}
 
 	@Override
-	public SideBarEntrySWT getCurrentEntry() {
-		return (SideBarEntrySWT) super.getCurrentEntry();
+	public SideBarEntrySWT getSelectedEntry() {
+		return (SideBarEntrySWT) super.getSelectedEntry();
+	}
+	
+	private void
+	setMenuEntry(
+		SideBarEntrySWT		me )
+	{
+		activeMenuEntry = me;
+		
+		if ( me != null ){
+			
+			lastMenuEntry = me;
+		}
+	}
+	
+	@Override
+	public SideBarEntrySWT 
+	getMenuEntry(
+		boolean activeOnly )
+	{
+		return( activeOnly?activeMenuEntry:lastMenuEntry );
 	}
 
 	@Override
