@@ -680,7 +680,7 @@ public abstract class TableViewSWT_Common
 						if (filter.nextText != null
 								&& (force || !filter.nextText.equals(filter.text))) {
 							filter.text = filter.nextText;
-							filter.checker.filterSet(filter.text);
+							filter.checker.filterSet(filter.text, filter.regex);
 							tv.refilter();
 						}
 					}
@@ -1048,6 +1048,22 @@ public abstract class TableViewSWT_Common
 				});
 			}
 			
+			MenuItem itemRowDetails = new MenuItem(menu, SWT.PUSH);
+			Messages.setLanguageText(itemRowDetails,
+					"MyTorrentsView.menu.row.details");
+
+			itemRowDetails.addListener(SWT.Selection, new Listener() {
+				@Override
+				public void handleEvent(Event e) {
+					TableRowCore focusedRow = tv.getFocusedRow();
+					if (focusedRow == null || focusedRow.isRowDisposed()) {
+						focusedRow = tv.getRow(0);
+					}
+					String tableID = tv.getTableID();
+					new TableRowViewer(tv.getDataSourceType(), tableID, focusedRow );
+				}
+			});
+			
 			MenuItem itemChangeTable = new MenuItem(menu, SWT.PUSH);
 			Messages.setLanguageText(itemChangeTable,
 					"MyTorrentsView.menu.editTableColumns");
@@ -1069,7 +1085,7 @@ public abstract class TableViewSWT_Common
 		}
 		String tableID = tv.getTableID();
 		new TableColumnSetupWindow(tv.getDataSourceType(), tableID, column, focusedRow,
-				TableStructureEventDispatcher.getInstance(tableID)).open();
+				TableStructureEventDispatcher.getInstance(tableID), false);
 	}
 
 
@@ -1215,19 +1231,29 @@ public abstract class TableViewSWT_Common
 			itemPrefSize.addListener(SWT.Selection, e -> Utils.execSWTThread(() -> {
 				column.setPreferredWidth(-1);
 
-				tv.runForAllRows(new TableGroupRowRunner() {
-					@Override
-					public void run(TableRowCore row) {
-						row.fakeRedraw( column.getName());
-					}
-				});
+				int done =
+					tv.runForAllRows(new TableGroupRowRunner() {
+						@Override
+						public void run(TableRowCore row) {
+							row.fakeRedraw( column.getName());
+						}
+					});
 
 				int pref = column.getPreferredWidth();
 
+				if ( done == 0 ){
+					
+					pref = Math.max( column.getMinWidth(), column.getPreferredHeaderWidth());
+				}
+				
 				if ( pref != -1 ){
 
 					column.setWidth( pref );
 				}
+				
+					// we need this to avoid scroll bars getting confused for some reason...
+				
+				TableStructureEventDispatcher.getInstance(tv.getTableID()).tableStructureChanged(false, null);
 			}));
 
 			new MenuItem(menu, SWT.SEPARATOR);
